moose_to_snomed = {
    "heart_myocardium": {
        "ID": "74281007", # checked
        "name": "Myocardium structure" # checked
    },
    "heart_atrium_left": {
        "ID": "82471001", # checked
        "name": "Left atrial structure" # checked
    },
    "heart_atrium_right": {
        "ID": "73829009", # checked
        "name": "Right atrial structure" # checked
    },
    "heart_ventricle_left": {
        "ID": "87878005", # checked
        "name": "Left ventricular structure" # checked
    },
    "heart_ventricle_right": {
        "ID": "53085002", # checked
        "name": "Right ventricular structure" # checked
    },
    "aorta": {
        "ID": "15825003", # checked
        "name": "Aortic structure" # checked
    },
    "iliac_artery_left": {
        "ID": "721077009", # checked
        "name": "Structure of left iliac artery" # checked
    },
    "iliac_artery_right": {
        "ID": "721035009", # checked
        "name": "Structure of right iliac artery" # checked
    },
    "iliac_vena_left": {
        "ID": "244411005", # checked
        "name": "Iliac vein structure" # checked
    },
    "iliac_vena_right": {
        "ID": "244411005", # checked
        "name": "Iliac vein structure" # checked
    },
    "inferior_vena_cava": {
        "ID": "64131007", # checked
        "name": "Inferior vena cava structure" # checked
    },
    "portal_splenic_vein": {
        "ID": "35819009", # checked
        "name": "Structure of splenic vein" # checked
    },
    "pulmonary_artery": {
        "ID": "81040000", # checked
        "name": "Pulmonary artery structure" # checked
    },
    "colon": {
        "ID": "71854001", # checked
        "name": "Colon structure" # checked
    },
    "duodenum": {
        "ID": "38848004", # checked
        "name": "Duodenum structure" # checked
    },
    "esophagus": {
        "ID": "32849002", # checked
        "name": "Esophageal structure" # checked
    },
    "small_bowel": {
        "ID": "30315005", # checked
        "name": "Small intestinal structure" # checked
    },
    "autochthon_left": {
        "ID": "44947003", # checked, but might need reevaluation
        "name": "Structure of erector spinae muscle" # checked, but might need reevaluation
    },
    "autochthon_right": {
        "ID": "44947003", # checked, but might need reevaluation
        "name": "Structure of erector spinae muscle" # checked, but might need reevaluation
    },
    "gluteus_maximus_left": {
        "ID": "206007", # checked, but might need reevaluation
        "name": "Structure of gluteus maximus muscle" # checked, but might need reevaluation
    },
    "gluteus_maximus_right": {
        "ID": "206007", # checked, but might need reevaluation
        "name": "Structure of gluteus maximus muscle" # checked, but might need reevaluation
    },
    "gluteus_medius_left": {
        "ID": "78333006", # checked, but might need reevaluation
        "name": "Structure of gluteus medius muscle" # checked, but might need reevaluation
    },
    "gluteus_medius_right": {
        "ID": "78333006", # checked, but might need reevaluation
        "name": "Structure of gluteus medius muscle" # checked, but might need reevaluation
    },
    "gluteus_minimus_left": {
        "ID": "75297007", # checked, but might need reevaluation
        "name": "Structure of gluteus minimus muscle" # checked, but might need reevaluation
    },
    "gluteus_minimus_right": {
        "ID": "75297007", # checked, but might need reevaluation
        "name": "Structure of gluteus minimus muscle" # checked, but might need reevaluation
    },
    "iliopsoas_left": {
        "ID": "68455001", # checked, but might need reevaluation
        "name": "Structure of iliopsoas muscle" # checked, but might need reevaluation
    },
    "iliopsoas_right": {
        "ID": "68455001", # checked, but might need reevaluation
        "name": "Structure of iliopsoas muscle" # checked, but might need reevaluation
    },
    "adrenal_gland_left": {
        "ID": "12003004", # checked
        "name": "Left adrenal gland" # checked
    },
    "adrenal_gland_right": {
        "ID": "29392005", # checked
        "name": "Right adrenal gland" # checked
    },
    "bladder": {
        "ID": "89837001", # checked
        "name": "Urinary bladder structure" # checked
    },
    "brain": {
        "ID": "12738006", # checked
        "name": "Brain structure" # checked
    },
    "gallbladder": {
        "ID": "28231008", # checked
        "name": "Gallbladder structure" # checked
    },
    "kidney_left": {
        "ID": "18639004", # checked
        "name": "Left kidney structure" # checked
    },
    "kidney_right": {
        "ID": "9846003", # checked
        "name": "Right kidney structure" # checked
    },
    "liver": {
        "ID": "10200004", # checked
        "name": "Liver structure" # checked
    },
    "lung_lower_lobe_left": {
        "ID": "41224006", # checked
        "name": "Lower lobe of left lung" # checked
    },
    "lung_lower_lobe_right": {
        "ID": "266005", # checked
        "name": "Lower lobe of right lung" # checked
    },
    "lung_middle_lobe_right": {
        "ID": "72481006", # checked
        "name": "Middle lobe of right lung" # checked
    },
    "lung_upper_lobe_left": {
        "ID": "44714003", # checked
        "name": "Upper lobe of left lung" # checked
    },
    "lung_upper_lobe_right": {
        "ID": "42400003", # checked
        "name": "Upper lobe of right lung" # checked
    },
    "pancreas": {
        "ID": "15776009", # checked
        "name": "Pancreatic structure" # checked
    },
    "spleen": {
        "ID": "78961009", # checked
        "name": "Splenic structure" # checked
    },
    "stomach": {
        "ID": "69695003", # checked
        "name": "Stomach structure" # checked
    },
    "thyroid_left": {
        "ID": "79163004", # checked
        "name": "Left lobe of thyroid gland" # checked
    },
    "thyroid_right": {
        "ID": "29565003", # checked
        "name": "Right lobe of thyroid gland" # checked
    },
    "trachea": {
        "ID": "44567001", # checked
        "name": "Trachea structure" # checked
    },
    "carpal_left": {
        "ID": "719623009", # checked, but might need reevaluation
        "name": "Bone structure of left carpus" # checked, but might need reevaluation
    },
    "carpal_right": {
        "ID": "719624003", # checked, but might need reevaluation
        "name": "Bone structure of right carpus" # checked, but might need reevaluation
    },
    "clavicle_left": {
        "ID": "720617006", # checked
        "name": "Bone structure of left clavicle" # checked
    },
    "clavicle_right": {
        "ID": "720616002", # checked
        "name": "Bone structure of right clavicle" # checked
    },
    "femur_left": {
        "ID": "1356755002", # checked
        "name": "Entire bone of left femur" # checked
    },
    "femur_right": {
        "ID": "1356756001", # checked
        "name": "Entire bone of right femur" # checked
    },
    "fibula_left": {
        "ID": "1017225004", # checked
        "name": "Entire left fibula" # checked
    },
    "fibula_right": {
        "ID": "1017224000", # checked
        "name": "Entire right fibula" # checked
    },
    "fingers_left": {
        "ID": "762870007", # checked, but might need reevaluation
        "name": "Bone structure of phalanx of left hand" # checked, but might need reevaluation
    },
    "fingers_right": {
        "ID": "762871006", # checked, but might need reevaluation
        "name": "Bone structure of phalanx of right hand" # checked, but might need reevaluation
    },
    "humerus_left": {
        "ID": "719460003", # checked
        "name": "Bone structure of left humerus" # checked
    },
    "humerus_right": {
        "ID": "719461004", # checked
        "name": "Bone structure of right humerus" # checked
    },
    "metacarpal_left": {
        "ID": "762008002", # checked, but might need reevaluation
        "name": "Structure of metacarpal bone of left hand" # checked, but might need reevaluation
    },
    "metacarpal_right": {
        "ID": "762009005", # checked, but might need reevaluation
        "name": "Structure of metacarpal bone of right hand" # checked, but might need reevaluation
    },
    "metatarsal_left": {
        "ID": "726438004", # checked, but might need reevaluation
        "name": "Structure of metatarsal bone of left foot" # checked, but might need reevaluation
    },
    "metatarsal_right": {
        "ID": "726439007", # checked, but might need reevaluation
        "name": "Structure of metatarsal bone of right foot" # checked, but might need reevaluation
    },
    "patella_left": {
        "ID": "734208001", # checked
        "name": "Bone structure of left patella" # checked
    },
    "patella_right": {
        "ID": "734210004", # checked
        "name": "Bone structure of right patella" # checked
    },
    "radius_left": {
        "ID": "719464007", # checked
        "name": "Bone structure of left radius" # checked
    },
    "radius_right": {
        "ID": "719465008", # checked
        "name": "Bone structure of right radius" # checked
    },
    "scapula_left": {
        "ID": "719627005", # checked
        "name": "Bone structure of left scapula" # checked
    },
    "scapula_right": {
        "ID": "719628000", # checked
        "name": "Bone structure of right scapula" # checked
    },
    "skull": {
        "ID": "118646007", # checked
        "name": "Entire bone of head" # checked
    },
    "tarsal_left": {
        "ID": "108371006", # checked, but might need reevaluation
        "name": "Bone structure of tarsus" # checked, but might need reevaluation
    },
    "tarsal_right": {
        "ID": "108371006", # checked, but might need reevaluation
        "name": "Bone structure of tarsus" # checked, but might need reevaluation
    },
    "tibia_left": {
        "ID": "719492002", # checked
        "name": "Bone structure of left tibia" # checked
    },
    "tibia_right": {
        "ID": "719491009", # checked
        "name": "Bone structure of right tibia" # checked
    },
    "toes_left": {
        "ID": "720642002", # checked, but might need reevaluation
        "name": "Bone structure of phalanx of left foot" # checked, but might need reevaluation
    },
    "toes_right": {
        "ID": "720641009", # checked, but might need reevaluation
        "name": "Bone structure of phalanx of right foot" # checked, but might need reevaluation
    },
    "ulna_left": {
        "ID": "719462006", # checked
        "name": "Bone structure of left ulna" # checked
    },
    "ulna_right": {
        "ID": "719463001", # checked
        "name": "Bone structure of right ulna" # checked
    },
    "rib_left_1": {
        "ID": "1354709002", # checked
        "name": "Bone structure of left first rib" # checked
    },
    "rib_left_2": {
        "ID": "1354712004", # checked
        "name": "Bone structure of left second rib" # checked
    },
    "rib_left_3": {
        "ID": "1354715002", # checked
        "name": "Bone structure of left third rib" # checked
    },
    "rib_left_4": {
        "ID": "1354718000", # checked
        "name": "Bone structure of left fourth rib" # checked
    },
    "rib_left_5": {
        "ID": "1354724006", # checked
        "name": "Bone structure of left fifth rib" # checked
    },
    "rib_left_6": {
        "ID": "1354726008", # checked
        "name": "Bone structure of left sixth rib" # checked
    },
    "rib_left_7": {
        "ID": "1354728009", # checked
        "name": "Bone structure of left seventh rib" # checked
    },
    "rib_left_8": {
        "ID": "1354733008", # checked
        "name": "Bone structure of left eighth rib" # checked
    },
    "rib_left_9": {
        "ID": "1354735001", # checked
        "name": "Bone structure of left ninth rib" # checked
    },
    "rib_left_10": {
        "ID": "1354737009", # checked
        "name": "Bone structure of left tenth rib" # checked
    },
    "rib_left_11": {
        "ID": "1354742001", # checked
        "name": "Bone structure of left eleventh rib" # checked
    },
    "rib_left_12": {
        "ID": "1354744000", # checked
        "name": "Bone structure of left twelfth rib" # checked
    },
    "rib_left_13": {
        "ID": "205460009", # checked
        "name": "Accessory rib" # checked
    },
    "rib_right_1": {
        "ID": "1354710007", # checked
        "name": "Bone structure of right first rib" # checked
    },
    "rib_right_2": {
        "ID": "1354713009", # checked
        "name": "Bone structure of right second rib" # checked
    },
    "rib_right_3": {
        "ID": "1354716001", # checked
        "name": "Bone structure of right third rib" # checked
    },
    "rib_right_4": {
        "ID": "1354719008", # checked
        "name": "Bone structure of right fourth rib" # checked
    },
    "rib_right_5": {
        "ID": "1354725007", # checked
        "name": "Bone structure of right fifth rib" # checked
    },
    "rib_right_6": {
        "ID": "1354727004", # checked
        "name": "Bone structure of right sixth rib" # checked
    },
    "rib_right_7": {
        "ID": "1354729001", # checked
        "name": "Bone structure of right seventh rib" # checked
    },
    "rib_right_8": {
        "ID": "1354734002", # checked
        "name": "Bone structure of right eighth rib" # checked
    },
    "rib_right_9": {
        "ID": "1354736000", # checked
        "name": "Bone structure of right ninth rib" # checked
    },
    "rib_right_10": {
        "ID": "1354738004", # checked
        "name": "Bone structure of right tenth rib" # checked
    },
    "rib_right_11": {
        "ID": "1354743006", # checked
        "name": "Bone structure of right eleventh rib" # checked
    },
    "rib_right_12": {
        "ID": "1354745004", # checked
        "name": "Bone structure of right twelfth rib" # checked
    },
    "rib_right_13": {
        "ID": "205460009", # checked
        "name": "Accessory rib" # checked
    },
    "sternum": {
        "ID": "302522007", # checked
        "name": "Entire sternum" # checked
    },
    "vertebra_C1": {
        "ID": "731658004", # checked
        "name": "Entire bone of atlas" # checked
    },
    "vertebra_C2": {
        "ID": "731673003", # checked
        "name": "Entire bone of axis" # checked
    },
    "vertebra_C3": {
        "ID": "731721006", # checked
        "name": "Entire bone of C3" # checked
    },
    "vertebra_C4": {
        "ID": "731650006", # checked
        "name": "Entire bone of C4" # checked
    },
    "vertebra_C5": {
        "ID": "731670000", # checked
        "name": "Entire bone of C5" # checked
    },
    "vertebra_C6": {
        "ID": "731668009", # checked
        "name": "Entire bone of C6" # checked
    },
    "vertebra_C7": {
        "ID": "731715007", # checked
        "name": "Entire bone of C7" # checked
    },
    "vertebra_T1": {
        "ID": "731692001", # checked
        "name": "Entire bone of T1" # checked
    },
    "vertebra_T2": {
        "ID": "731683004", # checked
        "name": "Entire bone of T2" # checked
    },
    "vertebra_T3": {
        "ID": "731648003", # checked
        "name": "Entire bone of T3" # checked
    },
    "vertebra_T4": {
        "ID": "731700006", # checked
        "name": "Entire bone of T4" # checked
    },
    "vertebra_T5": {
        "ID": "731685006", # checked
        "name": "Entire bone of T5" # checked
    },
    "vertebra_T6": {
        "ID": "731676006", # checked
        "name": "Entire bone of T6" # checked
    },
    "vertebra_T7": {
        "ID": "731689000", # checked
        "name": "Entire bone of T7" # checked
    },
    "vertebra_T8": {
        "ID": "731654002", # checked
        "name": "Entire bone of T8" # checked
    },
    "vertebra_T9": {
        "ID": "731711003", # checked
        "name": "Entire bone of T9" # checked
    },
    "vertebra_T10": {
        "ID": "731652003", # checked
        "name": "Entire bone of T10" # checked
    },
    "vertebra_T11": {
        "ID": "731656000", # checked
        "name": "Entire bone of T11" # checked
    },
    "vertebra_T12": {
        "ID": "731662005", # checked
        "name": "Entire bone of T12" # checked
    },
    "vertebra_L1": {
        "ID": "731695004", # checked
        "name": "Entire bone of L1" # checked
    },
    "vertebra_L2": {
        "ID": "731657009", # checked
        "name": "Entire bone of L2" # checked
    },
    "vertebra_L3": {
        "ID": "731669001", # checked
        "name": "Entire bone of L3" # checked
    },
    "vertebra_L4": {
        "ID": "731655001", # checked
        "name": "Entire bone of L4" # checked
    },
    "vertebra_L5": {
        "ID": "731680001", # checked
        "name": "Entire bone of L5" # checked
    },
    "vertebra_L6": {
        "ID": "82137000", # checked
        "name": "Entire bone of L6" # checked
    },
    "hip_left": {
        "ID": "287679003", # checked
        "name": "Left hip region structure" # checked
    },
    "hip_right": {
        "ID": "287579007", # checked
        "name": "Right hip region structure" # checked
    },
    "sacrum": {
        "ID": "699698002", # checked
        "name": "Sacrum" # checked
    },
    "skeletal_muscle": {
        "ID": "127954009", # checked
        "name": "Skeletal muscle structure" # checked
    },
    "subcutaneous_fat": {
        "ID": "67769002", # checked
        "name": "Subcutaneous adipose tissue" # checked
    },
    "visceral_fat": {
        "ID": "725273006", # checked, but might need reevaluation
        "name": "Structure of adipose tissue of abdomen" # checked, but might need reevaluation
    }
}